@@ -14,35 +14,26 @@ except ImportError:
 from lxml import etree
 from lxml.html import defs
 from lxml.html import fromstring, tostring, XHTML_NAMESPACE
-from lxml.html import _nons, _transform_result
+from lxml.html import xhtml_to_html, _transform_result
 
 try:
-    set
+    unichr
 except NameError:
-    # Python 3
-    from sets import Set as set
-
-try:
-    unichr = __builtins__['unichr']
-except (NameError, KeyError):
     # Python 3
     unichr = chr
-
 try:
-    unicode = __builtins__['unicode']
-except (NameError, KeyError):
+    unicode
+except NameError:
     # Python 3
     unicode = str
-
 try:
-    bytes = __builtins__['bytes']
-except (NameError, KeyError):
+    bytes
+except NameError:
     # Python < 2.6
     bytes = str
-
 try:
-    basestring = __builtins__['basestring']
-except (NameError, KeyError):
+    basestring
+except NameError:
     basestring = (str, bytes)
 
 
@@ -80,7 +71,7 @@ _css_import_re = re.compile(
 # All kinds of schemes besides just javascript: that can cause
 # execution:
 _javascript_scheme_re = re.compile(
-    r'\s*(?:javascript|jscript|livescript|vbscript|about|mocha):', re.I)
+    r'\s*(?:javascript|jscript|livescript|vbscript|data|about|mocha):', re.I)
 _substitute_whitespace = re.compile(r'\s+').sub
 # FIXME: should data: be blocked?
 
@@ -106,7 +97,8 @@ class Cleaner(object):
         Removes any ``<script>`` tags.
 
     ``javascript``:
-        Removes any Javascript, like an ``onclick`` attribute.
+        Removes any Javascript, like an ``onclick`` attribute. Also removes stylesheets
+        as they could contain Javascript.
 
     ``comments``:
         Removes any comments.
@@ -136,10 +128,15 @@ class Cleaner(object):
         Removes any form tags
 
     ``annoying_tags``:
-        Tags that aren't *wrong*, but are annoying.  ``<blink>`` and ``<marque>``
+        Tags that aren't *wrong*, but are annoying.  ``<blink>`` and ``<marquee>``
 
     ``remove_tags``:
-        A list of tags to remove.
+        A list of tags to remove.  Only the tags will be removed,
+        their content will get pulled up into the parent tag.
+
+    ``kill_tags``:
+        A list of tags to kill.  Killing also removes the tag's content,
+        i.e. the whole subtree, not just the tag itself.
 
     ``allow_tags``:
         A list of tags to include (default include all).
@@ -149,8 +146,11 @@ class Cleaner(object):
 
     ``safe_attrs_only``:
         If true, only include 'safe' attributes (specifically the list
-        from `feedparser
-        <http://feedparser.org/docs/html-sanitization.html>`_).
+        from the feedparser HTML sanitisation web site).
+
+    ``safe_attrs``:
+        A set of attribute names to override the default list of attributes
+        considered 'safe' (when safe_attrs_only=True).
 
     ``add_nofollow``:
         If true, then any <a> tags will have ``rel="nofollow"`` added to them.
@@ -167,6 +167,8 @@ class Cleaner(object):
         Note that this parameter might not work as intended if you do not
         make the links absolute before doing the cleaning.
 
+        Note that you may also need to set ``whitelist_tags``.
+
     ``whitelist_tags``:
         A set of tags that can be included with ``host_whitelist``.
         The default is ``iframe`` and ``embed``; you may wish to
@@ -191,8 +193,10 @@ class Cleaner(object):
     annoying_tags = True
     remove_tags = None
     allow_tags = None
+    kill_tags = None
     remove_unknown_tags = True
     safe_attrs_only = True
+    safe_attrs = defs.safe_attrs
     add_nofollow = False
     host_whitelist = ()
     whitelist_tags = set(['iframe', 'embed'])
@@ -234,10 +238,7 @@ class Cleaner(object):
             # ElementTree instance, instead of an element
             doc = doc.getroot()
         # convert XHTML to HTML
-        for el in doc.iter():
-            tag = el.tag
-            if isinstance(tag, basestring):
-                el.tag = _nons(tag)
+        xhtml_to_html(doc)
         # Normalize a case that IE treats <image> like <img>, and that
         # can confuse either this step or later steps.
         for el in doc.iter('image'):
@@ -246,23 +247,23 @@ class Cleaner(object):
             # Of course, if we were going to kill comments anyway, we don't
             # need to worry about this
             self.kill_conditional_comments(doc)
-        kill_tags = set()
+
+        kill_tags = set(self.kill_tags or ())
         remove_tags = set(self.remove_tags or ())
-        if self.allow_tags:
-            allow_tags = set(self.allow_tags)
-        else:
-            allow_tags = set()
+        allow_tags = set(self.allow_tags or ())
+
         if self.scripts:
             kill_tags.add('script')
         if self.safe_attrs_only:
-            safe_attrs = set(defs.safe_attrs)
+            safe_attrs = set(self.safe_attrs)
             for el in doc.iter():
                 attrib = el.attrib
                 for aname in attrib.keys():
                     if aname not in safe_attrs:
                         del attrib[aname]
         if self.javascript:
-            if not self.safe_attrs_only:
+            if not (self.safe_attrs_only and
+                    self.safe_attrs == defs.safe_attrs):
                 # safe_attrs handles events attributes itself
                 for el in doc.iter():
                     attrib = el.attrib
@@ -277,7 +278,7 @@ class Cleaner(object):
                 for el in _find_styled_elements(doc):
                     old = el.get('style')
                     new = _css_javascript_re.sub('', old)
-                    new = _css_import_re.sub('', old)
+                    new = _css_import_re.sub('', new)
                     if self._has_sneaky_javascript(new):
                         # Something tricky is going on...
                         del el.attrib['style']
@@ -305,8 +306,7 @@ class Cleaner(object):
             kill_tags.add(etree.ProcessingInstruction)
         if self.style:
             kill_tags.add('style')
-            for el in _find_styled_elements(doc):
-                del el.attrib['style']
+            etree.strip_attributes(doc, 'style')
         if self.links:
             kill_tags.add('link')
         elif self.style or self.javascript:
@@ -315,7 +315,8 @@ class Cleaner(object):
             for el in list(doc.iter('link')):
                 if 'stylesheet' in el.get('rel', '').lower():
                     # Note this kills alternate stylesheets as well
-                    el.drop_tree()
+                    if not self.allow_element(el):
+                        el.drop_tree()
         if self.meta:
             kill_tags.add('meta')
         if self.page_structure:
@@ -343,7 +344,7 @@ class Cleaner(object):
             remove_tags.add('form')
             kill_tags.update(('button', 'input', 'select', 'textarea'))
         if self.annoying_tags:
-            remove_tags.update(('blink', 'marque'))
+            remove_tags.update(('blink', 'marquee'))
 
         _remove = []
         _kill = []
@@ -371,12 +372,12 @@ class Cleaner(object):
                 el.tag = 'div'
             el.clear()
 
+        _kill.reverse() # start with innermost tags
         for el in _kill:
             el.drop_tree()
         for el in _remove:
             el.drop_tag()
 
-        allow_tags = self.allow_tags
         if self.remove_unknown_tags:
             if allow_tags:
                 raise ValueError(
@@ -387,12 +388,25 @@ class Cleaner(object):
             for el in doc.iter():
                 if el.tag not in allow_tags:
                     bad.append(el)
-            for el in bad:
-                el.drop_tag()
+            if bad:
+                if bad[0] is doc:
+                    el = bad.pop(0)
+                    el.tag = 'div'
+                    el.attrib.clear()
+                for el in bad:
+                    el.drop_tag()
         if self.add_nofollow:
             for el in _find_external_links(doc):
                 if not self.allow_follow(el):
-                    el.set('rel', 'nofollow')
+                    rel = el.get('rel')
+                    if rel:
+                        if ('nofollow' in rel
+                                and ' nofollow ' in (' %s ' % rel)):
+                            continue
+                        rel = '%s nofollow' % rel
+                    else:
+                        rel = 'nofollow'
+                    el.set('rel', rel)
 
     def allow_follow(self, anchor):
         """
@@ -497,7 +511,7 @@ clean_html = clean.clean_html
 ############################################################
 
 _link_regexes = [
-    re.compile(r'(?P<body>https?://(?P<host>[a-z0-9._-]+)(?:/[/\-_.,a-z0-9%&?;=~]*)?)', re.I),
+    re.compile(r'(?P<body>https?://(?P<host>[a-z0-9._-]+)(?:/[/\-_.,a-z0-9%&?;=~]*)?(?:\([/\-_.,a-z0-9%&?;=~]*\))?)', re.I),
     # This is conservative, but autolinking can be a bit conservative:
     re.compile(r'mailto:(?P<body>[a-z0-9._-]+@(?P<host>[a-z0-9_._]+[a-z]))', re.I),
     ]
@@ -527,7 +541,7 @@ def autolink(el, link_regexes=_link_regexes,
     host that matches one of the regular expressions in avoid_hosts
     (default localhost and 127.0.0.1).
 
-    If you pass in an element, the elements tail will not be
+    If you pass in an element, the element's tail will not be
     substituted, only the contents of the element.
     """
     if el.tag in avoid_elements:
