@@ -78,6 +78,13 @@
 #endif
 #endif
 
+#if PY_VERSION_HEX <= 0x03030000 && !(defined(CYTHON_PEP393_ENABLED) && CYTHON_PEP393_ENABLED)
+  #define PyUnicode_IS_READY(op)    (0)
+  #define PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
+  #define PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
+  #define PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
+#endif
+
 /* PySlice_GetIndicesEx() has wrong signature in Py<=3.1 */
 #if PY_VERSION_HEX >= 0x03020000
 #  define _lx_PySlice_GetIndicesEx(o, l, b, e, s, sl) PySlice_GetIndicesEx(o, l, b, e, s, sl)
@@ -172,7 +179,7 @@
 #ifndef LIBXML2_NEW_BUFFER
    typedef xmlBuffer xmlBuf;
 #  define xmlBufContent(buf) xmlBufferContent(buf)
-#  define xmlBufLength(buf) xmlBufferLength(buf)
+#  define xmlBufUse(buf) xmlBufferLength(buf)
 #endif
 
 /* libexslt 1.1.25+ support EXSLT functions in XPath */
@@ -232,6 +239,7 @@ long _ftol2( double dblSource ) { return _ftol( dblSource ); }
 #define _getNs(c_node) \
         (((c_node)->ns == 0) ? 0 : ((c_node)->ns->href))
 
+
 /* Macro pair implementation of a depth first tree walker
  *
  * Calls the code block between the BEGIN and END macros for all elements
